/**
 * Copyright 2009 Ashley Williams
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package acme;

import java.io.File;

import protoj.lang.ClassesArchive;
import protoj.lang.JavadocArchive;
import protoj.lang.ProjectArchive;
import protoj.lang.SourceArchive;
import protoj.lang.StandardProject;
import protoj.util.ArgRunnable;
import acme.core.AcmeCore;
import acme.util.AcmeUtil;

/**
 * Sets up a number of use-cases so that they can be tested with the AcmeSession
 * class.
 * 
 * @author Ashley Williams
 * 
 */
public final class AcmeProject {
	public static void main(String[] args) {
		new AcmeProject(args).dispatchCommands();
	}

	private StandardProject delegate;

	public AcmeProject(String[] args) {
		AcmeCore core = new AcmeCore(args);
		delegate = core.getProject();
		delegate.initJunit("16m");
		String acmeVersion = core.getAcmeVersion();
		String junitVersion = core.getJunitVersion();
		String projectName = "acme-" + acmeVersion;

		ClassesArchive classes = delegate.getArchiveFeature()
				.getClassesArchive();

		classes.addEntry(projectName, null, null, null);
		classes.getEntry(projectName).initConfig(
				new ArgRunnable<ClassesArchive>() {
					public void run(ClassesArchive archive) {
						archive.initExecutableJar();
					}
				});
		classes.initClasspathLib(projectName);
		classes.initIncludeArchives(projectName, "junit-" + junitVersion
				+ ".jar");
		String gpgOptions = "--armor --output %2$s --detach-sign %1$s";
		classes.initPublish(projectName, "/acme/acme-publish.xml", gpgOptions);

		String fooName = "foo";
		classes.addEntry(fooName, "foo-manifest", "**/*.txt", null);
		classes.getEntry(fooName).initConfig(new ArgRunnable<ClassesArchive>() {
			public void run(ClassesArchive archive) {
				archive.getCurrentAssembleTask().initManifest("bar", "bar");
			}
		});
		classes.initPublish(fooName, "/acme/foo-publish.xml", null);

		SourceArchive source = delegate.getArchiveFeature().getSourceArchive();
		source.addEntry("foosources", null, null, null);
		source.initIncludeArchives("foosources", "junit-" + junitVersion
				+ "-sources.jar");
		source.addEntry(projectName, null, null, null);

		JavadocArchive javadoc = delegate.getArchiveFeature()
				.getJavadocArchive();
		javadoc.addEntry("foojavadoc", null, null, null, "16m");
		javadoc.addEntry(projectName, null, null, null, "16m");

		delegate.getArchiveFeature().initProjectArchive("acme", projectName,
				null, null);
		delegate.getArchiveFeature().getProjectArchive().initConfig(
				new ArgRunnable<ProjectArchive>() {
					public void run(ProjectArchive feature) {
						feature.addFileSet(null, null,
								"part-private part-private/amber.txt", null);
					}
				});
		delegate.initUploadGoogleCode("acme");
		delegate.getCommandStore().addCommand("fail", "16m", new Runnable() {
			public void run() {
				failingTask();
			}
		}).initHelpString("causes an exception to be thrown");
		File publishUrl = new File(delegate.getLayout().getTargetDir(), "repo");
		delegate.getCommandStore().addCommand("create-file", "16m",
				new Runnable() {
					public void run() {
						createFile();
					}
				}).initHelpString(
				"creates a new file in response to a system property");
		delegate.getCommandStore().addCommand("message", "16m", new Runnable() {
			public void run() {
				System.out.println("acme message");
			}
		}).initHelpString("prints a message to stdout");
		delegate.initPublish("file://" + publishUrl.getAbsolutePath());
	}

	public void dispatchCommands() {
		delegate.getDispatchFeature().dispatchCommands();
	}

	public void createFile() {
		String argFile = System.getProperty("acme.file");
		AcmeUtil.createFile(argFile);
	}

	public void failingTask() {
		AcmeUtil.throwException();
	}
}
