package alien.core;

import protoj.lang.CompileFeature;
import protoj.lang.StandardProject;
import protoj.util.ArgRunnable;

/**
 * One of the project goals is to perform the compilation in-code, but this
 * means that at least that part of the project must be compiled first usually
 * by a shell script. But that's ok as long as the script keeps the amount of
 * code it compiles down to a minimum - in the case of alien.sh this means just
 * the AlienCore class.
 * <p>
 * But most of the use-cases are only defined in the AlienProject class and so
 * we have to get there somehow. Luckily we have just enough use-cases available
 * to us right here to achieve that goal:
 * <ol>
 * <li>compile command- this is used to compile the remainder of the project
 * including the target AlienProject class. A retrieve command is also available
 * in order to download any required libraries as defined in the the project ivy
 * or maven file into the lib/ directory.</li>
 * <li>bootstrap - this isn't a command for you to call, but it lets the
 * StandardProject pojo know that it's ok to call on the AlienProject class with
 * requests for any use-case that can't be found here.</li>
 * </ol>
 * 
 * 
 * @author Ashley Williams
 */
public final class AlienCore {
	/**
	 * The shell script enters here and must pass <code>-rootDir</code> and
	 * <code>-scriptName</code> init command options for example:
	 * 
	 * <pre>
	 * &quot;init --rootDir `pwd`/.. --scriptName alien.sh
	 * </pre>
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		AlienCore core = new AlienCore(args);
		core.dispatchCommands();
	}

	/**
	 * The top level protoj pojo.
	 */
	public final StandardProject project;

	/**
	 * Makes the compile use-case available and provides a link to the
	 * AlienProject class that in turn provides even more use-cases.
	 * 
	 * @param args
	 */
	public AlienCore(String[] args) {
		// the central domain object is created with the command line
		// arguments and some additional version information
		project = new StandardProject(args, "1.0");

		// enables configuration support with no resolving of ${x} placeholders
		// so that files from conf/profile get merged to the project root tree
		project.initDirConfig();

		// project resources can access the project code with velocity markup -
		// eg the maven and ivy files don't have to use hard-coded values
		project.getResourceFeature().getContext().put("alien", this);

		// enables jar downloading from the maven repository to the lib dir
		// using ivy - maven can also be used with a suitable pom.xml file
		project.getRetrieveFeature().initIvy("/alien/ivy.xml");

		// enables compilation with the specified amount of memory
		project.getCompileFeature().initConfig(false,
				new ArgRunnable<CompileFeature>() {
					public void run(CompileFeature feature) {
						feature.getJavacCompileTask().getCompileTask()
								.setMemoryMaximumSize("16m");
						feature.getJavacCompileTask().getCompileTask()
								.setSource("1.5");
					}
				});

		// provides the link to the main project class
		project.getDispatchFeature().initBootstrap("alien.AlienProject", "16m");
	}

	/**
	 * Dispatches command line arguments to the commands that were set up in the
	 * constructors of this class and of AlienProject.
	 */
	public void dispatchCommands() {
		project.getDispatchFeature().dispatchCommands();
	}

	/**
	 * This method is accessed from inside the ivy.xml and pom.xml resources,
	 * using velocity markup, instead of hard-coding the value.
	 * 
	 * @return
	 */
	public String getAlienVersion() {
		return "1.0";
	}

	/**
	 * This method is accessed from inside the ivy.xml and pom.xml resources,
	 * using velocity markup, instead of hard-coding the value.
	 * 
	 * @return
	 */
	public String getJunitVersion() {
		return "4.4";
	}

	/**
	 * The AlienProject instance will be adding an additional command to the
	 * standard project.
	 * 
	 * @return
	 */
	public StandardProject getProject() {
		return project;
	}

}
