package org.serverdemo.core;

import protoj.lang.CompileFeature;
import protoj.lang.StandardProject;
import protoj.util.ArgRunnable;

/**
 * This is the entry point of the virtual machine that handles commands related
 * to compilation. It is launched directly from the shell script and delegates
 * other commands to the main ServerDemoProject virtual machine.
 * 
 * @author Ashley Williams
 * 
 */
public final class ServerDemoCore {
	/**
	 * Called from serverdemo.sh.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		ServerDemoCore core = new ServerDemoCore(args);
		core.dispatchCommands();
	}

	/**
	 * See {@link #getProject()}.
	 */
	public final StandardProject project;

	/**
	 * Makes the compile use-case available and provides a link to the
	 * AlienProject class that in turn provides even more use-cases.
	 * 
	 * @param args
	 */
	public ServerDemoCore(String[] args) {
		// the central domain object is created with the command line
		// arguments and some additional version information
		project = new StandardProject(args, "1.0");

		// enables configuration support with resolving of ${x} placeholders
		// so that files from conf/profile get merged to the project root tree
		// with a combined properties file called conf/all.project.properties
		project.initDirConfig();

		// alternatively change the scp credentials below and then you can use
		// the "scpconf" command to apply a profile stored at "host" in the
		// "~/allprofiles/test" profile directory and use the scpconf command
		project.initScpConfig("configuser@host:~/allprofiles");

		// enables jar downloading from the maven repository to the lib dir
		project.getRetrieveFeature().initMaven("/org/serverdemo/pom.xml");

		// project resources can access the project code with velocity markup -
		// eg the maven and ivy files don't have to use hard-coded values
		project.getResourceFeature().getContext().put("serverdemo", this);

		// load time weaving is enabled for this project
		project.getDispatchFeature().initLoadTimeWeaving(null);

		// enables compilation for the given memory, the compile source, java
		// version and any specified cross compilation options
		project.getCompileFeature().initConfig(true,
				new ArgRunnable<CompileFeature>() {
					public void run(CompileFeature feature) {
						// all parameters here can be overridden with system
						// properties defined in properties files if any
						feature.initAjcCrossCompile(null, null, null);
						feature.initAjcSource(getJavaSpecificationVersion());
						feature.initAjcMemory("16m");
					}
				});

		// provides the link to the main project class
		project.getDispatchFeature().initBootstrap(
				"org.serverdemo.system.ServerDemoProject", "8m");
	}

	/**
	 * Dispatches command line arguments to the commands that arrived in the
	 * main method of this class.
	 */
	public void dispatchCommands() {
		project.getDispatchFeature().dispatchCommands();
	}

	/**
	 * The top level protoj pojo.
	 * 
	 * @return
	 */
	public StandardProject getProject() {
		return project;
	}

	/**
	 * This is the version of java that we are running in.
	 * 
	 * @return
	 */
	public String getJavaSpecificationVersion() {
		return System.getProperty("java.specification.version");
	}

}
