package org.serverdemo.system;

import java.io.File;
import java.io.IOException;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.HttpResponseException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.log4j.Logger;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Chmod;
import org.apache.tools.ant.taskdefs.Delete;
import org.apache.tools.ant.taskdefs.Expand;
import org.apache.tools.ant.types.Environment;
import org.apache.tools.ant.types.Environment.Variable;
import org.serverdemo.core.ServerDemoCore;

import protoj.lang.ArchiveFeature;
import protoj.lang.ClassesArchive;
import protoj.lang.ProjectArchive;
import protoj.lang.ProjectLayout;
import protoj.lang.PropertyInfo;
import protoj.lang.ResourceFeature;
import protoj.lang.ScpFeature;
import protoj.lang.StandardProject;
import protoj.util.AntTarget;
import protoj.util.ArgRunnable;
import protoj.util.CommandTask;

/**
 * The class demonstrates some of the ways that a jboss server instance can be
 * controlled. For example:
 * <ul>
 * <li>downloading third party artifacts from the maven repositories</li>
 * <li>extracting and customizing the jboss application server and configuring
 * permissions</li>
 * <li>starting the instance</li>
 * <li>stopping the instance</li>
 * <li>adding remote debugging</li>
 * <li>adding performance tuning options</li>
 * <li>adding load time weaving capabilities</li>
 * <li>providing custom binding properties</li>
 * <li>replacing ${var} tokens inside config files</li>
 * <li>creating an executable jar of the entire project</li>
 * </ul>
 * <p>
 * The implementation is not heavily refactored, since I believe a blunter,
 * linear style is more suited to tutorial code.
 * 
 * @author Ashley Williams
 * 
 */
public final class ServerDemoProject {
	/**
	 * Called from ServerDemoCore.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		new ServerDemoProject(args).dispatchCommands();
	}

	/**
	 * The underlying ProtoJ pojo.
	 */
	private StandardProject delegate;

	/**
	 * The location of the expanded jboss application server.
	 */
	private File jbossDir;

	/**
	 * The port used when debugging jboss.
	 */
	private PropertyInfo jbossDebugPort;

	/**
	 * The suspend value used when debugging jboss.
	 */
	private PropertyInfo jbossDebugSuspend;

	/**
	 * The flag used to determine whether or not to debug jboss.
	 */
	private PropertyInfo jbossDebug;

	/**
	 * The additional free-typed JAVA_OPTS configuration.
	 */
	private PropertyInfo jbossJavaOpts;

	/**
	 * The port used for jboss jnp service.
	 */
	private PropertyInfo jbossNamingPort;

	/**
	 * The ip address used by most jboss services to bind to.
	 */
	private PropertyInfo jbossBindAddress;

	/**
	 * The port used for http connections to jboss.
	 */
	private PropertyInfo jbossWebPort;

	/**
	 * The maximum amount of memory that the jboss vm may allocate.
	 */
	private PropertyInfo jbossMaxmem;

	/**
	 * The reduced functionality available in the bootstrap vm.
	 */
	private ServerDemoCore core;

	/**
	 * Configures the various features offered by ProtoJ.
	 * 
	 * @param args
	 */
	public ServerDemoProject(String[] args) {
		// contains the protoj StandardProject instance with all the standard
		// use-cases built in
		core = new ServerDemoCore(args);

		// a cached reference to the ProtoJ pojo
		delegate = core.getProject();

		// enables junit testing with the specified amount of memory
		delegate.initJunit("16m");

		ArchiveFeature archive = delegate.getArchiveFeature();

		// enables classes jar file creation with the specified jar name
		ClassesArchive classesArchive = archive.getClassesArchive();
		classesArchive.addEntry("serverdemo", null,
				"org/serverdemo/**/* META-INF/**/*", null);
		classesArchive.initClasspathLib("serverdemo");

		// enables creation of a self extracting project jar and
		// ensures the expanded jboss directory is added to it
		archive.initSelfExtractingArchive("serverdemo-dist", null,
				"serverdemo", null, null);
		archive.getProjectArchive().initConfig(
				new ArgRunnable<ProjectArchive>() {
					public void run(ProjectArchive projectArchive) {
						String includes = jbossDir.getName() + "/**";
						projectArchive.addFileSet("755", "755", includes, null);
					}
				});
		delegate.getCommandStore().addCommand("dist", "16m", new Runnable() {
			public void run() {
				delegate.getArchiveFeature().createSelfExtractingArchive(true,
						true, true);
			}
		}).initHelpString("creates a distribution of the serverdemo project");
		delegate.getCommandStore().addCommand("expand", "16m", new Runnable() {
			public void run() {
				expand();
			}
		}).initHelpString("expands jboss to the root directory");
		delegate.getCommandStore().addCommand("start", "12m", new Runnable() {
			public void run() {
				startServer();
			}
		}).initHelpString("starts the jboss default server");
		delegate.getCommandStore().addCommand("ping", "12m", new Runnable() {
			public void run() {
				pingServer(10, 10, true);
			}
		}).initHelpString("pings the jboss server to check status");
		delegate.getCommandStore().addCommand("stop", "8m", new Runnable() {
			public void run() {
				stopServer();
			}
		}).initHelpString("stops the jboss default server");
		delegate.getCommandStore().addCommand("mycmd", "32m", new Runnable() {
			public void run() {
				ArchiveFeature archive = delegate.getArchiveFeature();
				ScpFeature scpFeature = delegate.getScpFeature();

				// expand the untouched jboss
				expand();

				// make adjustments to jboss
				delegate.getDirconfFeature().configure("profile", true);

				// create our first party jars
				archive.getClassesArchive().createArchives();
				archive.getSourceArchive().createArchives();

				// create the executable jar of the whole project
				archive.createSelfExtractingArchive(true, true, true);

				// scp the executable jar to the release directory
				String toDir = "releasemgr@http://releasemachine:/opt/releases";
				scpFeature.copyClasses("serverdemo-dist", toDir, "");
				// see CompileCommand source for an example of how to
				// add command line options with jopt simple
			}
		}).initHelpResource("experimental command");

		jbossBindAddress = delegate.getPropertyStore().addInfo(
				"jboss.bind.address", "the jboss bind address");

		jbossNamingPort = delegate.getPropertyStore().addInfo(
				"serverdemo.jboss.naming.port", "the jboss jnp bootstrap port");

		jbossWebPort = delegate.getPropertyStore().addInfo(
				"serverdemo.jboss.web.port", "the jboss http connection port");

		jbossDebug = delegate.getPropertyStore().addInfo(
				"serverdemo.jboss.debug", Boolean.FALSE.toString(),
				Boolean.TRUE.toString(),
				"switch used to enable jboss debugging");

		jbossDebugPort = delegate.getPropertyStore().addInfo(
				"serverdemo.jboss.debug.port", "11161", "11161",
				"port used for debugging jboss");

		jbossDebugSuspend = delegate.getPropertyStore().addInfo(
				"serverdemo.jboss.debug.suspend", "n", "y",
				"whether or not to suspend when debugging jboss");

		jbossMaxmem = delegate.getPropertyStore().addInfo(
				"serverdemo.jboss.maxmemory", "512m", "",
				"the maximum amount of memory that the jboss vm may allocate");

		jbossJavaOpts = delegate.getPropertyStore().addInfo(
				"serverdemo.jboss.java.opts",
				"additional JAVA_OPTS not related to debugging or weaving");

		// when expanded this is the jboss root directory
		jbossDir = new File(delegate.getLayout().getRootDir(), "jboss-5.0.1.GA");
	}

	/**
	 * Dispatches command line arguments to the commands that arrived in the
	 * main method of this class.
	 */
	public void dispatchCommands() {
		delegate.getDispatchFeature().dispatchCommands();
	}

	/**
	 * Expands the jboss artifact as returned by {@link #getJbossZipName()}
	 * found directly under the source directory to the root directory. Since
	 * this is a large file it doesn't ship with this project and must instead
	 * be downloaded from the jboss website and place directly under the src
	 * directory.
	 * <p>
	 * During expansion all the shipped jboss server instances are deleted
	 * except for the one named "default". This instance is then modified with
	 * our custom files stored in the src/resources directory, using the ProtoJ
	 * ResourceFeature pojo.
	 * <p>
	 * Although we don't take advantage of the ability in serverdemo, ProtoJ
	 * classpath resources need not be static. They can contain velocity markup,
	 * which means they can refer to all methods and properties from this
	 * project instance.
	 * 
	 */
	public void expand() {
		ProjectLayout layout = delegate.getLayout();
		File srcDir = layout.getSrcDir();
		File jbossArtifact = new File(srcDir, getJbossZipName());

		// verifies that the jboss artifact has been downloaded
		if (!jbossArtifact.exists()) {
			StringBuilder builder = new StringBuilder();
			builder
					.append("Expected to find downloaded jboss artifact with this path:\n");
			builder.append(jbossArtifact.getAbsolutePath());
			builder
					.append("\nTry the following url that worked at the time of testing for version 5:\n");
			builder
					.append("http://sourceforge.net/project/downloading.php?group_id=22866&filename=jboss-5.0.1.GA.zip&a=50047482");
			builder.append("\nor the following url for version 6:\n");
			builder
					.append("http://sourceforge.net/project/downloading.php?group_id=22866&filename=jboss-5.0.1.GA-jdk6.zip&a=44097217");
			throw new RuntimeException(builder.toString());
		}

		AntTarget target = new AntTarget("serverdemo-project");
		target.initLogging(Project.MSG_INFO);

		// adds a task to expand the jboss zip file
		Expand expand = new Expand();
		target.addTask(expand);
		expand.setTaskName("expand");
		expand.setSrc(jbossArtifact);
		expand.setDest(layout.getRootDir());

		// adds a task to trim unwanted content
		addDelete(target, new File(jbossDir, "docs"));
		addDelete(target, new File(jbossDir, "server/all"));
		addDelete(target, new File(jbossDir, "server/minimal"));
		addDelete(target, new File(jbossDir, "server/standard"));
		addDelete(target, new File(jbossDir, "server/web"));

		// adds a task to update permsissions
		Chmod chmod = new Chmod();
		target.addTask(chmod);
		chmod.setTaskName("update-permission");
		chmod.setDir(jbossDir);
		chmod.setIncludes("**/*");
		chmod.setPerm("700");

		target.execute();

		// modifies the default server with our custom content and ensures
		// velocity markup is handled during the copy
		ResourceFeature resourceFeature = delegate.getResourceFeature();
		String prefix = "/org/serverdemo/jboss";
		resourceFeature.filterResourceToDir(prefix,
				"server/default/deploy/properties-service.xml", jbossDir);
		resourceFeature.filterResourceToDir(prefix,
				"server/default/conf/jboss-service.xml", jbossDir);
	}

	/**
	 * Returns "jboss-5.0.1.GA-jdk6" if this is a java 6 vm,
	 * "jboss-5.0.1.GA.zip" if it is version 5. There are two versions of the
	 * jboss application server and we need to know which one to expand. Helper
	 * for {@link #expand()}.
	 * 
	 * @return
	 */
	private String getJbossZipName() {
		String name;
		String version = core.getJavaSpecificationVersion();
		if (version.contains("6")) {
			name = "jboss-5.0.1.GA-jdk6.zip";
		} else if (version.contains("5")) {
			name = "jboss-5.0.1.GA.zip";
		} else {
			throw new RuntimeException("demo only executes on java 5 or java 6");
		}
		return name;
	}

	/**
	 * Adds an ant task that deletes the specfied directory. Helper for
	 * {@link #expand()}.
	 * 
	 * @param target
	 * @param dir
	 */
	private void addDelete(AntTarget target, File dir) {
		Delete delete = new Delete();
		target.addTask(delete);
		delete.setTaskName("trim-" + dir.getName());
		delete.setDir(dir);
	}

	/**
	 * Starts the default server instance. The process is spawned so that this
	 * method will continue immediately, but we will have no clue whether or not
	 * the process was successful. Therefore it subsequently makes a limited
	 * number of attempts to ping the jboss server for activity.
	 * <p>
	 * The jboss JAVA_OPTS string is also built up and applied based on any
	 * configured user properties. These are shipped in the
	 * jboss.opts.properties file, but they can go anywhere.
	 */
	public void startServer() {
		// creates a command that executes the jboss run shell script
		File jbossBinDir = new File(jbossDir, "bin");
		String runScript = new File(jbossBinDir, "run.").getCanonicalPath();
		String bindAddress = jbossBindAddress.getMandatoryValue();
		String commandLine = "-c default -b " + bindAddress;
		CommandTask task = new CommandTask(jbossBinDir, runScript, commandLine,
				"\"Serverdemo Jboss Application Server\"");
		task.initLogging(Project.MSG_INFO);
		task.initSpawn(true);

		StringBuilder allJavaOpts = new StringBuilder();

		// builds up load time weaving config which is mandatory for serverdemo
		File libDir = delegate.getLayout().getLibDir();
		File weaverJar = new File(libDir, "aspectjweaver.jar");
		allJavaOpts.append("-javaagent:" + weaverJar.getAbsolutePath());

		// builds up optional remote debug config based on user properties
		boolean debug = jbossDebug.getBooleanValue();
		if (debug) {
			String port = jbossDebugPort.getValue();
			String suspend = jbossDebugSuspend.getValue();
			String debugConfig = " -Xdebug -Xrunjdwp:transport=dt_socket,address=%s,server=y,suspend=%s";
			allJavaOpts.append(String.format(debugConfig, port, suspend));
		}

		// applies the configured jboss maximum memory setting
		allJavaOpts.append(" -Xmx" + jbossMaxmem.getMandatoryValue());

		// adds optional additional free-typed config based on user properties
		String javaOpts = jbossJavaOpts.getValue();
		if (javaOpts.length() > 0) {
			allJavaOpts.append(" ");
			allJavaOpts.append(javaOpts);
		}

		// applies the resultant JAVA_OPTS environment variable
		Variable var = new Environment.Variable();
		var.setKey("JAVA_OPTS");
		var.setValue(allJavaOpts.toString());
		task.getExecTask().addEnv(var);

		String raw = "starting server with\ncommand line options: %s\njava options: %s";
		String message = String.format(raw, commandLine, allJavaOpts);
		delegate.getLogger().info(message);

		task.execute();

		// waits for up to four minutes to check for the jboss http port
		if (!pingServer(24, 10, true)) {
			throw new RuntimeException(
					"jboss didn't start in a reasonable amount of time");
		}
		delegate.getLogger().info("server started ok");
	}

	/**
	 * Stops the jboss server if it is running. The jnp address used is
	 * calculated by coordinating with the user properties.
	 * <p>
	 * The properties are shipped in the jboss.bindings.properties file, but
	 * they can go anywhere.
	 */
	public void stopServer() {
		File jbossBinDir = new File(jbossDir, "bin");
		String runScript = new File(jbossBinDir, "shutdown.")
				.getCanonicalPath();

		// compose the address and stop the server
		String namingPort = jbossNamingPort.getMandatoryValue();
		String bindAddress = jbossBindAddress.getMandatoryValue();
		String address = "jnp://" + bindAddress + ":" + namingPort;
		delegate.getLogger().info("stopping server at " + address);
		CommandTask task = new CommandTask(jbossBinDir, runScript, "-s "
				+ address, null);
		task.initLogging(Project.MSG_INFO);
		task.initSpawn(true);
		task.execute();

		// blocks for a while to check for the jboss http port
		if (!pingServer(3, 10, false)) {
			throw new RuntimeException(
					"jboss didn't stop in a reasonable amount of time");
		}

		delegate.getLogger().info("server stopped ok");
	}

	/**
	 * Checks the health of the jboss server by pinging its default web page the
	 * specified number of times. Can be configured to check if the server has
	 * started or stopped.
	 * 
	 * @param maxAttempts
	 *            the number of pings that should be tried
	 * @param waitSeconds
	 *            how long to wait between pings
	 * @param checkStarted
	 *            true if you want to check if the server has started, false if
	 *            you want to check the server has stopped
	 * @return true if the server was seen to have started or stopped within the
	 *         allowable time, or false otherwise
	 */
	public boolean pingServer(int maxAttempts, int waitSeconds,
			boolean checkStarted) {
		boolean conditionMet = false;
		int currentAttempt = 0;
		HttpClient httpclient = new DefaultHttpClient();

		Logger logger = delegate.getLogger();
		try {
			String webPort = jbossWebPort.getMandatoryValue();
			String address = "http://localhost:" + webPort;
			logger.info(String.format("pinging %s %s times", address,
					maxAttempts));
			HttpGet httpget = new HttpGet(address);

			// this response handler merely captures the status code
			ResponseHandler<Integer> responseHandler = new ResponseHandler<Integer>() {
				public Integer handleResponse(final HttpResponse response)
						throws HttpResponseException, IOException {
					int statusCode = response.getStatusLine().getStatusCode();
					return statusCode;
				}
			};

			// pings until the correct response code is returned
			// and while there are still attempts remaining
			while (!conditionMet && currentAttempt < maxAttempts) {
				currentAttempt++;
				Thread.sleep(waitSeconds * 1000L);
				try {
					Integer statusCode = httpclient.execute(httpget,
							responseHandler);
					logger.info("got status " + statusCode);
					conditionMet = statusCode.equals(200) && checkStarted;
				} catch (Exception e) {
					logger.info("address not available");
					// if we are checking for a stopped server then success
					// otherwise we must carry on checking
					conditionMet = !checkStarted;
				}
			}
		} finally {
			httpclient.getConnectionManager().shutdown();
		}

		return conditionMet;
	}

}
